/*
    $Id: file_input.h 53 2009-11-01 18:03:28Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _FILE_INPUT_H
#define _FILE_INPUT_H

#include <iosfwd>
#include <stdexcept>
#include <string>

#include <boost/algorithm/string.hpp>

#include "io.h"
#include "astrotime.h"
#include "coordinates.h"

using namespace boost::numeric;


class file_input {
public:
	typedef size_t size_type;
private:
	size_type m_line;

	void do_parse( std::istream& stm );
	static std::string strip_spaces( const std::string& str );
	const static std::string::size_type payload_col;

public:
	struct object {
		unsigned int hd;
		std::string  name;
		Coord<equ>   coordinates;
		std::string  sclass;
	};

	class failure:
		public std::runtime_error {
	private:
		size_type m_line;
	public:
		failure( const char* what, size_type line );
		size_type line() const;
	};
public:
	file_input();
	virtual ~file_input();

	void operator() ( std::istream& stm );

	size_type line() const throw() ;
public:
	virtual void on_comment( const std::string& line ) = 0;
	virtual void on_parameter( const std::string& key, const std::string& value ) = 0;
	virtual void on_object( const object& obj ) = 0;
	virtual void on_background( const Date& date, const Time& time ) = 0;
	virtual void on_normal( const Date& date, const Time& time ) = 0;
	virtual void on_flux( const Date& date, const Time& time ) = 0;
	virtual void on_measurements( const Date& date, const Time& time, const ublas::vector<double>& mean, const ublas::vector<double>& vars, const ublas::vector<double>& rho1, const ublas::vector<double>& rho2 ) = 0;
};

namespace std {
	std::istream& operator>>( std::istream& stm, file_input::object& obj );
}

#endif // _FILE_INPUT_H
