/*
    $Id: file_output.cpp 53 2009-11-01 18:03:28Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/io/ios_state.hpp>

#include "coordinates.h"
#include "file_output.h"
#include "integrals.h"
#include "indices.h"

file_output::file_output( std::ostream& stm ): m_stm(stm) {

}
file_output::~file_output() {

}
void file_output::comment( const std::string& comment ) {
	m_stm << comment << std::endl;
}
void file_output::normal( const Date& date, const Time& time, double mz ) {
	boost::io::ios_all_saver asaver( m_stm );
	m_stm << "M " << date << " " << time << " Normal " << std::fixed << std::setprecision(3) << mz << std::endl;
}
void file_output::atmosphere( const Date& date, const Time& time, const integrals::atmospheric_struct& atmosphere ) {
	static datum skipped;
	boost::io::ios_all_saver asaver( m_stm );
	m_stm << "A " << date << " " << time << " "
		<< std::setprecision(3)
		<< std::fixed << atmosphere.fseeing << " " << skipped << " "
		<< std::scientific << atmosphere.moment0 << " " << skipped << " "
		<< std::fixed << atmosphere.altieff << " " << skipped << " " << atmosphere.isangle << " "
		<< std::scientific << atmosphere.moment2 << " "
		<< std::fixed << atmosphere.ticonst << std::endl;
}
void file_output::profile( const Date& date, const Time& time, const std::string& method, double cn2seeing, double chi2, const ublas::vector<double>& cn2, const ublas::vector<double>& ecn2, const ublas::vector<double>& grid ){
	assert( cn2.size() == ecn2.size() );
	assert( cn2.size() == grid.size() );

	boost::io::ios_all_saver asaver( m_stm );
	m_stm << "T " << date << " " << time << " " << method << " "
		<< std::setw(2) << cn2.size() << " " << std::setprecision(2)
		<< std::fixed << chi2 << " " << cn2seeing;
	for( ublas::vector<double>::size_type i = 0; i < cn2.size(); i++)
		m_stm << " " << std::fixed << grid(i)
			<< std::scientific  << " " << cn2(i) << " " << std::sqrt(ecn2(i));
	m_stm << std::endl;
}
void file_output::residual( const Date& date, const Time& time, const std::string& method, const ublas::vector<double>& o_csi ) {
	boost::io::ios_all_saver asaver( m_stm );
	m_stm << "R " << date << " " << time << " " << method << " "
		<< std::setprecision(3) << std::fixed;
	for( ublas::vector<double>::size_type i = 0; i < o_csi.size(); i++ )
		m_stm << "  " << o_csi(i);
	m_stm << std::endl;
}
void file_output::fluxes( const  Date& date, const Time& time, const ublas::vector<datum>& result_flux ) {
	boost::io::ios_all_saver asaver( m_stm );
	assert( result_flux.size() >= 4 );
	m_stm << "F " << date << " " << time << " " << std::fixed << result_flux(0) << ' ' << result_flux(1) << ' ' << result_flux(2) << ' ' << result_flux(3) << std::endl;
}
void file_output::indices( const  Date& date, const Time& time, const ublas::vector<datum>& result_indx, const ublas::vector<datum>& result_desi ) {
	boost::io::ios_all_saver asaver( m_stm );
	assert( result_desi.size() >= 4 );
	assert( result_indx.size() >= 10 );
	m_stm << "I " << date << " " << time << std::indic << std::fixed << result_indx(0) << result_indx(1) << result_indx(2) << result_indx(3)
		<< result_indx(4) << result_indx(5) << result_indx(6) << result_indx(7) << result_indx(8)
		<< result_indx(9) << result_desi(0) << result_desi(1) << result_desi(2) << result_desi(3) << std::endl;
}