/*
    $Id: file_output.h 53 2009-11-01 18:03:28Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _FILE_OUTPUT_H
#define _FILE_OUTPUT_H

#include <iosfwd>
#include <string>

#include <boost/format.hpp>
#include <boost/io/ios_state.hpp>
#include <boost/numeric/ublas/vector.hpp>

#include "integrals.h"

using namespace boost::numeric;

class file_output {
public:
	typedef size_t size_type;
private:
	std::ostream& m_stm;
public:
	file_output( std::ostream& stm );
	virtual ~file_output();

	void comment( const std::string& comment );
	void normal( const Date& date, const Time& time, double mz );
	void atmosphere( const Date& date, const Time& time, const integrals::atmospheric_struct& atmosphere );
	void profile( const Date& date, const Time& time, const std::string& method, double cn2seeing, double chi2, const ublas::vector<double>& cn2, const ublas::vector<double>& ecn2, const ublas::vector<double>& grid );
	void residual( const Date& date, const Time& time, const std::string& method, const ublas::vector<double>& o_csi );
	void fluxes( const  Date& date, const Time& time, const ublas::vector<datum>& result_flux );
	void indices( const  Date& date, const Time& time, const ublas::vector<datum>& result_indx, const ublas::vector<datum>& result_desi );

	virtual void fingerprint() = 0;

	std::ostream& operator()() { return m_stm; }
};

#endif // _FILE_OUTPUT_H
