/*
    $Id: indices.h 53 2009-11-01 18:03:28Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _INDICES_H
#define _INDICES_H

#include <string>
#include <iostream>
#include <iomanip>
#include <stdexcept>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>

#include "typesdef.h"
#include "io.h"

using namespace boost::numeric::ublas;

template<class MT, class VT> inline void matrix_push_row( MT& m, const VT& v ) {
	typedef typename MT::size_type size_type;
	size_type size_i = m.size1();
	size_type size_j = m.size2();

	assert( size_j == v.size() );

	m.resize( size_i + 1, size_j );
	row( m, size_i ) = v;
}

class indices {
private:
	double microexposure;// = 1.0;             // in milliseconds
	vector<double> deadtime;
	vector<double> nonpois;
	vector<double> scatter;
	vector<double> backgrd;
	matrix<double> flux;
	matrix<double> inds;
	matrix<double> desi;

	class failure:
		public std::runtime_error {
	public:
		failure( const std::string& what ): std::runtime_error( what ) {
		}
	};
public:
	indices();
	~indices();

	void calculation( const vector<double>& cmean, const vector<double>& cvars, const vector<double>& crho1, const vector<double>& crho2 );

	void set_counters( const vector<double>&, const vector<double>&, const vector<double>& );
	void set_background( const vector<double>& );
	void set_microexposure( double );

	const matrix<double>& fluxes() const {
		return flux;
	}
	const matrix<double>& index() const {
		return inds;
	}
	const matrix<double>& indesis() const {
		return desi;
	}
	vector<datum> result_fluxes() const {
		return result(flux);
	}
	vector<datum> result_indices() const {
		return result(inds);
	}
	vector<datum> result_indesis() const {
		return result(desi);
	}
	void reset() throw();
private:
	vector<datum> result( const matrix< double >& ) const;
};

#endif // _INDICES_H
