/*
    $Id: integrals.h 53 2009-11-01 18:03:28Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _INTEGRALS_H
#define _INTEGRALS_H

#include <string>
#include <stdexcept>

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>

#include "typesdef.h"
#include "weif.h"

using namespace boost::numeric::ublas;

class integrals{
public:
	struct atmospheric_struct {
		atmospheric_struct(datum, datum, datum, datum, datum, datum);

		datum fseeing;
		datum isangle;
		datum altieff;
		datum ticonst;
		datum moment0;
		datum moment2;
	};
	typedef atmospheric_struct result_type;

	class failure:
		public std::runtime_error {
	public:
		failure( const std::string& what ): std::runtime_error( what ) {
		}
	};
private:
	double         m_lambda_eff;
	unsigned int   m_nscind;
	matrix<double> m_dcf;

	std::string    m_spectrum;
public:
	integrals();
	~integrals();

	void coefficients( const std::string&, const Weights& );
	result_type atmospheric( const matrix<double>&, const matrix<double>&, double );

private:
	datum result( const vector<double>&  );
};

#endif // _INTEGRALS_H
