/*
    $Id$
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iosfwd>
#include <iomanip>

#include <boost/io/ios_state.hpp>

#include "io.h"

namespace std {
	static const int iword_index = std::ios_base::xalloc();
	std::ostream& indic( std::ostream& stm ){
		stm.iword(iword_index) = true;
		return stm;
	}
	std::ostream& operator<<( std::ostream& stm, const datum& m ){
		boost::io::ios_flags_saver fsaver( stm );

		if( std::ostream::sentry( stm ) ){
			int w = ( stm.flags() & std::ios::scientific )? 8 : 5;
			w = ( stm.iword(iword_index) )? w + 2 : w;
			int p = ( stm.iword(iword_index) )? 4 : std::max( 0, 4 - std::max( 1, (int)ceil(std::log10( m.mean ) ) ) );
			stm << std::setprecision(p) << std::setw(w) << m.mean << " "
				<< std::fixed << std::setprecision(3) << std::setw(7) << m.errs;
		}
		return stm;
	}
}