/*
    $Id: profile.h 53 2009-11-01 18:03:28Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PROFILE_H
#define _PROFILE_H

#include <string>
#include <stdexcept>

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/symmetric.hpp>

#include "weif.h"

using namespace boost::numeric::ublas;

class restoration {
public:
	struct profile_struct {
		std::string cn2method;
		vector<double> cn2;
		vector<double> ecn2;
		double chi2;
		double cn2seeing;
		vector<double> o_csi;
	};
	typedef profile_struct result_type;

	class failure:
		public std::runtime_error {
	public:
		failure( const std::string& what ): std::runtime_error( what ) {
		}
	};
private:
	vector<double> m_userzcn2;
public:
	template<class VE> restoration( const VE& grid ): m_userzcn2( grid ) {}
	~restoration();
	result_type calcn2( const matrix<double>&, double, const Weights& ) const;
	template<class VE> void setzcn2( const VE& grid ) {
		m_userzcn2 = grid;
	}
	const vector<double>& zcn2() const { return m_userzcn2; }

private:
	void covariance( const matrix<double>&, vector<double>&, symmetric_matrix< double, upper >& ) const;
	double nnls_solve( const matrix< double >&, const matrix< double >&, vector< double >&, vector< double >&, vector< double >& ) const;
};

#endif // _PROFILE_H
