/*
    $Id: spectral_factory.cpp 56 2009-11-01 18:19:52Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/algorithm/string.hpp>
#include <boost/filesystem/operations.hpp>

#include "spectral_factory.h"

spectral_factory::spectral_factory(const boost::filesystem::path& wd, const boost::filesystem::path& response):
	m_wdir(wd),
	m_response(response),
	m_basis_grid( wf::lg_generator(0.0, 0.04, 0.25), 50 ) {

	if( ! (boost::filesystem::exists( wd ) && boost::filesystem::is_directory( wd )) )
		throw std::runtime_error( wd.string() + " is not a directory or doesn't exists");
	if( ! (boost::filesystem::exists( response ) && !boost::filesystem::is_directory( response )) )
		throw std::runtime_error( response.string() + " is not a regular file or doesn't exists");
}
spectral_factory::~spectral_factory() {

}
const boost::filesystem::path spectral_factory::get_spectrum_filename( const std::string& spectype ) const {
	std::string spectral_type( spectype );
	boost::to_lower( spectral_type );
	boost::filesystem::path spect( m_wdir / (spectral_type + ".sp") );
	return spect;
}
const spectral_factory::weight_type spectral_factory::spectral( const std::string& spectype, const ublas::vector<double>& inner_diam, const ublas::vector<double>& outer_diam ) {
	weight_type basis_functions( 4, m_basis_grid );
	//FIXME: Spectral_data
	boost::filesystem::path spectral_file = get_spectrum_filename( spectype );
	if( ! (boost::filesystem::exists( spectral_file ) && !boost::filesystem::is_directory( spectral_file )) )
		throw std::runtime_error( spectral_file.string() + " is not a regular file or doesn't exists");

	Spectral_data spd( m_response.string(), "", spectral_file.string() );
	basis_functions.calculation( spd, inner_diam, outer_diam );
	return basis_functions;
}
