/*
    $Id: typesdef.h 53 2009-11-01 18:03:28Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _TYPESDEF_H
#define _TYPESDEF_H
#include <vector>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/symmetric.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>

class datum {
public:
    double  mean;
    double  errs;
    datum( double m = 0.0, double e = 0.0 ) : mean(m), errs(e) {}
};

using namespace boost::numeric::ublas;

template<class OP, class E> BOOST_UBLAS_INLINE typename vector_unary_traits<E, OP>::result_type apply_to_all (const vector_expression<E> &e, const OP& op = OP() ) {
    typedef typename vector_unary_traits<E, OP>::expression_type expression_type;
    return expression_type (e ());
}

namespace functor {
    template <class T> class inv {
        public:
        typedef T value_type;
        typedef T result_type;
        inv() { }
        static  result_type apply( const value_type& x) {
            return ( x != value_type(0) )? value_type(1)/x : value_type(0);
        }
    };
    template <class T> class exp {
        public:
        typedef T value_type;
        typedef T result_type;
        exp() { }
        static  result_type apply( const value_type& x) {
            return std::exp( x );
        }
    };
    template <class T> class log {
        public:
        typedef T value_type;
        typedef T result_type;
        log() { }
        static  result_type apply( const value_type& x) {
            return std::log( x );
        }
    };
//FIXME:
    template <class T> class lambertW {
        public:
        typedef T value_type;
        typedef T result_type;
        lambertW() { }
        static  result_type apply( const value_type& x) {
            int i = 0;
            double w0 = 1;
            double w1 = 0;
            double ew;
            while( i++ < 20 && std::abs(w0-w1) > 1e-13 ) {
                w0 = w1;
                ew = std::exp(w0);
                w1 =  (w0*w0*ew + x)/ew/(1.0+w0);
            }
            return w1;
        }
    };
    template <class T> class sqrt {
        public:
        typedef T value_type;
        typedef T result_type;
        sqrt() { }
        static  result_type apply( const value_type& x) {
            return std::sqrt(x);
        }
    };
}

template<class ME> matrix_vector_slice< const ME > diag_elements( const ME& x ){
    typename ME::size_type r = std::min( x.size1(), x.size2() );
    slice s(0, 1, r);
    return matrix_vector_slice< const ME >( x, s, s );
}

template< int p, class T > vector<T> mass_order( const vector<T>&x ) {
    vector<T> y( x.size()*(x.size()+1)/2);
    subrange( y, 0, x.size() ) = x;
    unsigned int k = x.size();
    for( unsigned int i = 0; i < x.size()-1; i++ )
        for( unsigned int j = i+1; j < x.size(); j++ ) {
            if( p == 1 ) y(k++) = x(i);
            if( p == 2 ) y(k++) = x(j);
        }
    return y;
}

template<class T> vector<T> mass_order_prod( const vector<T>& x ) {
    return  element_prod( mass_order<1>( x ), mass_order<2>( x ) );
}

template<class T> vector<T> mass_order_summ( const vector<T>& x ) {
    return  mass_order<1>( x ) + mass_order<2>( x );
}

#endif // _TYPESDEF_H
